@@ -1,4 +1,4 @@
-# $Id: Server.pm,v 1.1 2005/04/10 21:07:12 joern Exp $
+# $Id: Server.pm,v 1.2 2005/04/15 21:09:51 joern Exp $
 
 #-----------------------------------------------------------------------
 # Copyright (C) 2002-2005 Jrn Reder <joern AT zyn.de>.
@@ -12,12 +12,15 @@ package Event::RPC::Server;
 
 use Event::RPC;
 use Event::RPC::Message;
+use Event::RPC::Connection;
+use Event::RPC::LogConnection;
 
 use Carp;
 use strict;
 use IO::Socket::INET;
 use Sys::Hostname;
 
+sub get_host			{ shift->{host}				}
 sub get_port			{ shift->{port}				}
 sub get_name			{ shift->{name}				}
 sub get_loop			{ shift->{loop}				}
@@ -37,7 +40,10 @@ sub get_ssl_passwd_cb		{ shift->{ssl_passwd_cb}		}
 sub get_auth_required		{ shift->{auth_required}		}
 sub get_auth_passwd_href	{ shift->{auth_passwd_href}		}
 sub get_listeners_started	{ shift->{listeners_started}		}
+sub get_connection_hook		{ shift->{connection_hook}		}
+sub get_auto_reload_modules	{ shift->{auto_reload_modules}		}
 
+sub set_host			{ shift->{host}			= $_[1]	}
 sub set_port			{ shift->{port}			= $_[1]	}
 sub set_name			{ shift->{name}			= $_[1]	}
 sub set_loop			{ shift->{loop}			= $_[1]	}
@@ -57,6 +63,8 @@ sub set_ssl_passwd_cb		{ shift->{ssl_passwd_cb}	= $_[1]	}
 sub set_auth_required		{ shift->{auth_required}	= $_[1]	}
 sub set_auth_passwd_href	{ shift->{auth_passwd_href}	= $_[1]	}
 sub set_listeners_started	{ shift->{listeners_started}	= $_[1]	}
+sub set_connection_hook		{ shift->{connection_hook}	= $_[1]	}
+sub set_auto_reload_modules	{ shift->{auto_reload_modules}	= $_[1]	}
 
 my $INSTANCE;
 sub instance { $INSTANCE }
@@ -64,13 +72,15 @@ sub instance { $INSTANCE }
 sub new {
 	my $class = shift;
 	my %par = @_;
-	my  ($port, $classes, $name, $logger, $start_log_listener) =
-	@par{'port','classes','name','logger','start_log_listener'};
+	my  ($host, $port, $classes, $name, $logger, $start_log_listener) =
+	@par{'host','port','classes','name','logger','start_log_listener'};
 	my  ($ssl, $ssl_key_file, $ssl_cert_file, $ssl_passwd_cb) =
 	@par{'ssl','ssl_key_file','ssl_cert_file','ssl_passwd_cb'};
 	my  ($auth_required, $auth_passwd_href, $loop) =
 	@par{'auth_required','auth_passwd_href','loop'};
-	
+	my  ($client_hook, $auto_reload_modules) =
+	@par{'client_hook','auto_reload_modules'};
+
 	$name ||= "Event-RPC-Server";
 	
 	if ( not $loop ) {
@@ -90,6 +100,7 @@ sub new {
 	}
 
 	my $self = bless {
+		host			=> $host,
 		port			=> $port,
 		name			=> $name,
 		classes			=> $classes,
@@ -105,12 +116,16 @@ sub new {
 		auth_required		=> $auth_required,
 		auth_passwd_href	=> $auth_passwd_href,
 
+		auto_reload_modules	=> $auto_reload_modules,
+		client_hook		=> $client_hook,
+
 		rpc_socket		=> undef,
 		loaded_classes		=> {},
 		objects			=> {},
 		logging_clients		=> {},
 		clients_connected	=> 0,
 		listeners_started	=> 0,
+		log_clients_connected	=> 0,
 	}, $class;
 
 	$INSTANCE = $self;
@@ -132,9 +147,13 @@ sub DESTROY {
 sub setup_listeners {
 	my $self = shift;
 
-	my $port = $self->get_port;
+	#-- Listener options
+	my $host      = $self->get_host;
+	my $port      = $self->get_port;
+	my @LocalHost = $host ? ( LocalHost => $host ) : ();
+	$host ||= "*";
 
-	#-- load event loop manager
+	#-- get event loop manager
 	my $loop = $self->get_loop;
 	
 	#-- setup rpc listener
@@ -147,8 +166,10 @@ sub setup_listeners {
 
 		$rpc_socket = IO::Socket::SSL->new (
 			Listen    	=> SOMAXCONN,
+			@LocalHost,
 			LocalPort 	=> $port,
 			Proto     	=> 'tcp',
+			ReuseAddr       => 1,
 			SSL_verify_mode => 0x00,
 			SSL_key_file	=> $self->get_ssl_key_file,
 			SSL_cert_file	=> $self->get_ssl_cert_file,
@@ -157,8 +178,10 @@ sub setup_listeners {
 	} else {
 		$rpc_socket = IO::Socket::INET->new (
 			Listen    => SOMAXCONN,
+			@LocalHost,
 			LocalPort => $port,
 			Proto     => 'tcp',
+			ReuseAddr => 1,
 		) or die "can't start RPC listener: $!";
 	}
 
@@ -172,9 +195,9 @@ sub setup_listeners {
 	);
 
 	if ( $self->get_ssl ) {
-		$self->log ("Started SSL RPC listener on port $port");
+		$self->log ("Started SSL RPC listener on port $host:$port");
 	} else {
-		$self->log ("Started RPC listener on port $port");
+		$self->log ("Started RPC listener on $host:$port");
 	}
 
 	# setup log listener
@@ -182,7 +205,9 @@ sub setup_listeners {
 		my $log_socket = IO::Socket::INET->new (
 			Listen    => SOMAXCONN,
 			LocalPort => $port + 1,
+			@LocalHost,
 			Proto     => 'tcp',
+			ReuseAddr => 1,
 		) or die "can't start log listener: $!";
 
 		$loop->add_io_watcher (
@@ -192,7 +217,7 @@ sub setup_listeners {
 			desc	=> "log listener port ".($port+1),
 		);
 
-		$self->log ("Started log listener on port ".($port+1));
+		$self->log ("Started log listener on $host:".($port+1));
 	}
 
 	$self->set_listeners_started(1);
@@ -208,7 +233,7 @@ sub start {
 
 	my $loop = $self->get_loop;
 
-	$self->log ("Enter main loop using $loop");
+	$self->log ("Enter main loop using ".ref($loop));
 
 	$loop->enter;
 
@@ -220,6 +245,7 @@ sub start {
 sub stop {
 	my $self = shift;
 
+use Carp; Carp::cluck();
 	$self->get_loop->leave;
 	
 	1;
@@ -231,7 +257,7 @@ sub accept_new_client {
 
 	my $client_socket = $rpc_socket->accept or return;
 
-	Event::RPC::Server::Connection->new ($self, $client_socket);
+	Event::RPC::Connection->new ($self, $client_socket);
 
 	$self->set_clients_connected ( 1 + $self->get_clients_connected );
 
@@ -245,11 +271,10 @@ sub accept_new_log_client {
 	my $client_socket = $log_socket->accept or return;
 
 	my $log_client =
-		Event::RPC::Server::LogConnection->new($self, $client_socket);
+		Event::RPC::LogConnection->new($self, $client_socket);
 
 	$self->set_log_clients_connected ( 1 + $self->get_log_clients_connected );
 	$self->get_logging_clients->{$log_client->get_cid} = $log_client;
-
 	$self->get_logger->add_fh($client_socket);
 
 	$self->log(2, "New log client connected");
@@ -261,7 +286,7 @@ sub load_class {
 	my $self = shift;
 	my ($class) = @_;
 
-	Event::RPC::Server::Connection->new ($self)->load_class($class);
+	Event::RPC::Connection->new ($self)->load_class($class);
 
 	return $class;
 }
@@ -342,567 +367,6 @@ sub print_object_register {
 	1;
 }
 
-package Event::RPC::Server::Connection;
-
-use Carp;
-use Socket;
-
-my $CONNECTION_ID;
-
-sub get_cid			{ shift->{cid}				}
-sub get_sock			{ shift->{sock}				}
-sub get_server			{ shift->{server}			}
-sub get_watcher			{ shift->{watcher}			}
-
-sub get_classes			{ shift->{server}->{classes}		}
-sub get_loaded_classes		{ shift->{server}->{loaded_classes}	}
-sub get_objects			{ shift->{server}->{objects}		}
-sub get_client_objects		{ shift->{client_objects}		}
-
-sub set_watcher			{ shift->{watcher}		= $_[1]	}
-
-sub get_message			{ shift->{message}			}
-sub get_is_authorized		{ shift->{is_authorized}		}
-sub get_auth_user		{ shift->{auth_user}			}
-
-sub set_message			{ shift->{message}		= $_[1]	}
-sub set_is_authorized		{ shift->{is_authorized}	= $_[1]	}
-sub set_auth_user		{ shift->{auth_user}		= $_[1]	}
-
-sub new {
-	my $class = shift;
-	my  ($server, $sock) = @_;
-
-	my $cid = ++$CONNECTION_ID;
-	
-	my $self = bless {
-		cid     		=> $cid,
-		sock    		=> $sock,
-		server  		=> $server,
-		is_authorized		=> (!$server->get_auth_required),
-		auth_user		=> "",
-		watcher 		=> undef,
-		message 		=> undef,
-		client_objects		=> {},
-	}, $class;
-
-	if ( $sock ) {
-		$self->log (2,
-			"Got new RPC connection. Connection ID is $cid"
-		);
-		$self->{watcher} = $self->get_server->get_loop->add_io_watcher (
-			fh   => $sock,
-			poll => 'r',
-			cb   => sub { $self->input; 1 },
-			desc => "rpc client cid=$cid",
-		);
-	}
-	
-	return $self;
-}
-
-sub disconnect {
-	my $self = shift;
-
-	close $self->get_sock;
-	$self->get_server->get_loop->del_io_watcher($self->get_watcher);
-	$self->set_watcher(undef);
-
-	my $server = $self->get_server;
-
-	$server->set_clients_connected ( $self->get_server->get_clients_connected - 1 );
-
-	foreach my $oid ( keys %{$self->get_client_objects} ) {
-		$server->deregister_object($oid);
-	}
-
-	$self->log(2, "Client disconnected");
-
-	1;
-}
-
-sub log {
-	my $self = shift;
-
-	my ($level, $msg);
-	if ( @_ == 2 ) {
-		($level, $msg) = @_;
-	} else {
-		($msg) = @_;
-		$level = 1;
-	}
-
-	$msg = "cid=".$self->get_cid.": $msg";
-	
-	return $self->get_server->log ($level, $msg);
-}
-
-sub input {
-	my $self = shift;
-	my ($e) = @_;
-
-	my $server  = $self->get_server;
-	my $message = $self->get_message;
-
-	if ( not $message ) {
-		$message = Event::RPC::Message->new ($self->get_sock);
-		$self->set_message($message);
-	}
-
-	my $request = eval { $message->read } || '';
-	my $error = $@;
-
-	return if not defined $request and not $error;
-
-	$self->set_message(undef);
-
-	return $self->disconnect
-		if $request eq "DISCONNECT\n" or
-		   $error =~ /DISCONNECTED/;
-
-	my ($cmd, $rc);
-	$cmd = $request->{cmd} if not $error;
-	
-	if ( $error ) {
-		$self->log ("Unexpected error on incoming RPC call: $@");
-		$rc = {
-			ok  => 0,
-			msg => "Unexpected error on incoming RPC call: $@",
-		};
-
-	} elsif ( $cmd eq 'auth' ) {
-		$rc = $self->authorize_user ($request);
-
-	} elsif ( $server->get_auth_required && !$self->get_is_authorized ) {
-		$rc = {
-			ok  => 0,
-			msg => "Authorization required",			
-		};
-
-	} elsif ( $cmd eq 'new' ) {
-		$rc = $self->create_new_object ($request);
-
-	} elsif ( $cmd eq 'exec' ) {
-		$rc = $self->execute_object_method ($request);
-
-	} elsif ( $cmd eq 'classes_list' ) {
-		$rc = $self->get_classes_list ($request);
-
-	} elsif ( $cmd eq 'class_info' ) {
-		$rc = $self->get_class_info ($request);
-
-	} elsif ( $cmd eq 'client_destroy' ) {
-		$rc = $self->object_destroyed_on_client ($request);
-
-	} else {
-		$self->log ("Unknown request command '$cmd'");
-		$rc = {
-			ok  => 0,
-			msg => "Unknown request command '$cmd'",
-		};
-	}
-
-	$message->write($rc) and return;
-
-	my $watcher;
-	$watcher = $self->get_server->get_loop->add_io_watcher (
-		fh	=> $self->get_sock,
-		poll	=> 'w',
-		cb	=> sub {
-		    $self->get_server->get_loop->del_io_watcher($watcher)
-		    	if $message->write;
-		    1;
-		},
-	);
-
-	1;
-}
-
-sub authorize_user {
-	my $self = shift;
-	my ($request) = @_;
-	
-	my $user = $request->{user};
-	my $pass = $request->{pass};
-	
-	my $auth_passwd_href = $self->get_server->get_auth_passwd_href;
-	my $server_pass = $auth_passwd_href->{$user} || '';
-
-	if ( $server_pass eq $pass ) {
-		$self->set_auth_user($user);
-		$self->set_is_authorized(1);
-		$self->log("User '$user' successfully authorized");
-		return {
-			ok  => 1,
-			msg => "Credentials Ok",
-		};
-	} else {
-		$self->log("Illegal credentials for user '$user'");
-		return {
-			ok  => 0,
-			msg => "Illegal credentials",
-		};
-	}
-}
-
-sub create_new_object {
-	my $self = shift;
-	my ($request) = @_;
-
-	# Let's create a new object
-	my $class_method = $request->{method};
-	my $class = $class_method;
-	$class =~ s/::[^:]+$//;
-	$class_method =~ s/^.*:://;
-
-	# check if access to this class/method is allowed
-	if ( not defined $self->get_classes->{$class}->{$class_method} or
-	     $self->get_classes->{$class}->{$class_method} ne '_constructor' ) {
-		$self->log ("Illegal constructor access to $class->$class_method");
-		return {
-			ok  => 0,
-			msg => "Illegal constructor access to $class->$class_method"
-		};
-
-	}
-	
-	# load the class if not done yet
-	$self->load_class($class);
-
-	# resolve object params
-	$self->resolve_object_params ($request->{params});
-
-	# ok, the class is there, let's execute the method
-	my $object = eval {
-		$class->$class_method (@{$request->{params}})
-	};
-
-	# report error
-	if ( $@ ) {
-		$self->log ("Error: can't create object ".
-			    "($class->$class_method): $@");
-		return {
-			ok  => 0,
-			msg => $@,
-		};
-	}
-
-	# register object
-	$self->get_server->register_object ($object, $class);
-	$self->get_client_objects->{"$object"} = 1;
-
-	# log and return
-	$self->log (5,
-		"Created new object $class->$class_method with oid '$object'",
-	);
-
-	return {
-		ok  => 1,
-		oid => "$object",
-	};
-}
-
-sub load_class {
-	my $self = shift;
-	my ($class) = @_;
-	
-	my $mtime;
-	my $load_class_info = $self->get_loaded_classes->{$class};
-
-	if ( not $load_class_info or
-	     ( $mtime = (stat($load_class_info->{filename}))[9])
-		> $load_class_info->{mtime} ) {
-	
-		if ( not $load_class_info->{filename} ) {
-			my $filename;
-			my $rel_filename = $class;
-			$rel_filename =~ s!::!/!g;
-			$rel_filename .= ".pm";
-
-			foreach my $dir ( @INC ) {
-				$filename = "$dir/$rel_filename", last
-					if -f "$dir/$rel_filename";
-			}
-
-			croak "File for class '$class' not found"
-				if not $filename;
-			
-			$load_class_info->{filename} = $filename;
-			$load_class_info->{mtime} = 0;
-		}
-	
-		$mtime ||= 0;
-
-		$self->log (3, "Class '$class' ($load_class_info->{filename}) changed on disk. Reloading...")
-			if $mtime > $load_class_info->{mtime};
-
-		do $load_class_info->{filename};
-
-		if ( $@ ) {
-			$self->log ("Can't load class '$class': $@");
-			$load_class_info->{mtime} = 0;
-
-			return {
-				ok  => 0,
-				msg => "Can't load class $class: $@",
-			};
-
-		} else {
-			$self->log (3, "Class '$class' successfully loaded");
-			$load_class_info->{mtime} = time;
-		}
-	}
-	
-	$self->log (5, "filename=".$load_class_info->{filename}.
-		    ", mtime=".$load_class_info->{mtime} );
-
-	$self->get_loaded_classes->{$class} ||= $load_class_info;
-
-	1;
-}
-
-sub execute_object_method {
-	my $self = shift;
-	my ($request) = @_;
-
-	# Method call of an existent object
-	my $oid = $request->{oid};
-	my $object_entry = $self->get_objects->{$oid};
-	my $method = $request->{method};
-
-	if ( not defined $object_entry ) {
-		# object does not exists
-		$self->log ("Illegal access to unknown object with oid=$oid");
-		return {
-			ok  => 0,
-			msg => "Illegal access to unknown object with oid=$oid"
-		};
-
-	}
-	
-	my $class = $object_entry->{class};
-	if ( not defined $self->get_classes->{$class}->{$method} ) {
-		# illegal access to this method
-		$self->log ("Illegal access to $class->$method");
-		return {
-			ok  => 0,
-			msg => "Illegal access to $class->$method"
-		};
-
-	}
-	
-	# (re)load the class if not done yet
-	$self->load_class($class);
-
-	# resolve object params
-	$self->resolve_object_params ($request->{params});
-
-	# ok, try executing the method
-	my @rc = eval {
-		$object_entry->{object}->$method (@{$request->{params}})
-	};
-
-	# report error
-	if ( $@ ) {
-		$self->log ("Error: can't call '$method' of object ".
-			    "with oid=$oid: $@");
-		return {
-			ok  => 0,
-			msg => $@,
-		};
-	}
-	
-	# log
-	$self->log (4, "Called method '$method' of object ".
-		       "with oid=$oid");
-
-	# check if objects are returned by this method
-	# and register them in our internal object table
-	# (if not already done yet)
-	my $key;
-	foreach my $rc ( @rc ) {
-		if ( ref ($rc) and ref ($rc) !~ /ARRAY|HASH|SCALAR/ ) {
-			# returns a single object
-			$self->log (4, "Method returns object: $rc");
-			$key = "$rc";
-			$self->get_client_objects->{$key} = 1;
-			$self->get_server->register_object($rc, ref $rc);
-			$rc = $key;
-
-		} elsif ( ref $rc eq 'ARRAY' ) {
-			# possibly returns a list of objects
-			# make a copy, otherwise the original object references
-			# will be overwritten
-			my @val = @{$rc};
-			$rc = \@val;
-			foreach my $val ( @val ) {
-				if ( ref ($val) and ref ($val) !~ /ARRAY|HASH|SCALAR/ ) {
-					$self->log (4, "Method returns object lref: $val");
-					$key = "$val";
-					$self->get_client_objects->{$key} = 1;
-					$self->get_server->register_object($val, ref $val);
-					$val = $key;
-				}
-			}
-		} elsif ( ref $rc eq 'HASH' ) {
-			# possibly returns a hash of objects
-			# make a copy, otherwise the original object references
-			# will be overwritten
-			my %val = %{$rc};
-			$rc = \%val;
-			foreach my $val ( values %val ) {
-				if ( ref ($val) and ref ($val) !~ /ARRAY|HASH|SCALAR/ ) {
-					$self->log (4, "Method returns object href: $val");
-					$key = "$val";
-					$self->get_client_objects->{$key} = 1;
-					$self->get_server->register_object($val, ref $val);
-					$val = $key;
-				}
-			}
-		}
-	}
-
-	# return rc
-	return {
-		ok => 1,
-		rc => \@rc,
-	};
-}
-
-sub object_destroyed_on_client {
-	my $self = shift;
-	my ($request) = @_;
-
-	$self->log(5, "Object with oid=$request->{oid} destroyed on client");
-
-	delete $self->get_client_objects->{$request->{oid}};
-	$self->get_server->deregister_object($request->{oid});
-
-	return {
-		ok => 1
-	};
-}
-
-sub get_classes_list {
-	my $self = shift;
-	my ($request) = @_;
-
-	my @classes = keys %{$self->get_classes};
-	
-	return {
-		ok      => 1,
-		classes => \@classes,
-	}
-}
-
-sub get_class_info {
-	my $self = shift;
-	my ($request) = @_;
-
-	my $class = $request->{class};
-	
-	if ( not defined $self->get_classes->{$class} ) {
-		$self->log ("Unknown class '$class'");
-		return {
-			ok  => 0,
-			msg => "Unknown class '$class'"
-		};
-	}
-	
-	$self->log (4, "Class info for '$class' requested");
-
-	return {
-		ok           => 1,
-		methods      => $self->get_classes->{$class},
-	};
-}
-
-sub resolve_object_params {
-	my $self = shift;
-	my ($params) = @_;
-	
-	my $key;
-	foreach my $par ( @{$params} ) {
-		if ( defined $self->get_classes->{ref($par)} ) {
-			$key = ${$par};
-			$key = "$key";
-			croak "unknown object with key '$key'"
-				if not defined $self->get_objects->{$key};
-			$par = $self->get_objects->{$key}->{object};
-		}
-	}
-	
-	1;
-}
-
-
-package Event::RPC::Server::LogConnection;
-
-use Carp;
-use Socket;
-
-my $LOG_CONNECTION_ID;
-
-sub get_cid			{ shift->{cid}				}
-sub get_sock			{ shift->{sock}				}
-sub get_server			{ shift->{server}			}
-
-sub get_watcher			{ shift->{watcher}			}
-sub set_watcher			{ shift->{watcher}		= $_[1]	}
-
-sub new {
-	my $class = shift;
-	my ($server, $sock) = @_;
-
-	my $cid = ++$LOG_CONNECTION_ID;
-	
-	my $self = bless {
-		cid     => $cid,
-		sock    => $sock,
-		server  => $server,
-		watcher => undef,
-	}, $class;
-
-	$self->{watcher} = $server->get_loop->add_io_watcher(
-		fh   => $sock,
-		poll => 're',
-		cb   => sub { $self->input; 1 },
-		desc => "log reader $cid",
-	);
-
-	$self->get_server->log (2,
-		"Got new logger connection. Connection ID is $cid"
-	);
-
-	return $self;
-}
-
-sub disconnect {
-	my $self = shift;
-
-	my $sock = $self->get_sock;
-	$self->get_server->get_logger->remove_fh($sock);
-	$self->get_server->get_loop->del_io_watcher($self->get_watcher);
-	$self->set_watcher(undef);
-	close $sock;
-
-	$self->get_server->set_log_clients_connected ( $self->get_server->get_log_clients_connected - 1 );
-	delete $self->get_server->get_logging_clients->{$self->get_cid};
-	$self->get_server->log(2, "Log client disconnected");
-
-	1;
-}
-
-sub input {
-	my $self = shift;
-
-	my $sock = $self->get_sock;
-	$self->disconnect if eof($sock);
-	<$sock>;
-	
-	1;
-}
-
 1;
 
 __END__
@@ -929,19 +393,23 @@ Event::RPC::Server - Simple API for event driven RPC servers
       },
 
       #-- Optional arguments
-      name               => "Test server",
-      logger             => Event::RPC::Logger->new(),
-      start_log_listener => 1,
+      name                => "Test server",
+      logger              => Event::RPC::Logger->new(),
+      start_log_listener  => 1,
 
-      ssl                => 1
-      ssl_key_file       => "server.key",
-      ssl_cert_file      => "server.crt",
-      ssl_passwd_cb      => sub { "topsecret" },
+      ssl                 => 1
+      ssl_key_file        => "server.key",
+      ssl_cert_file       => "server.crt",
+      ssl_passwd_cb       => sub { "topsecret" },
 
-      auth_required      => 1,
-      auth_passwd_href   => { $user => Event::RPC->crypt($user,$pass) },
+      auth_required       => 1,
+      auth_passwd_href    => { $user => Event::RPC->crypt($user,$pass) },
 
-      loop               => Event::RPC::Loop::Event->new(),
+      loop                => Event::RPC::Loop::Event->new(),
+      
+      host                => "localhost",
+      auto_reload_modules => 1,
+      connection_hook     => sub { ... },
   );
 
   $server->start;
@@ -962,11 +430,13 @@ it, or take control over the main loop on your own if you like
 General information about the architecture of Event::RPC driven
 applications is collected in the Event::RPC manpage.
 
-The following documentation describes mainly the options passed to the
-new() constructor of Event::RPC::Server, divided into several topics.
-
 =head1 CONFIGURATION OPTIONS
 
+All options described here may be passed to the new() constructor of
+Event::RPC::Server. As well you may set or modify them using set_OPTION style
+mutators, but not after start() or setup_listeners() was called!
+All options may be read using get_OPTION style accessors.
+
 =head2 REQUIRED OPTIONS
 
 If you just pass the required options listed beyond you have
@@ -1179,7 +649,7 @@ so be careful enabling the log listener in critical environments.
 =head2 MAINLOOP OPTIONS
 
 Event::RPC derived it's name from the fact that it follows the event
-driven paradigma. There are several toolkits for Perl which allow
+driven paradigm. There are several toolkits for Perl which allow
 event driven software development. Event::RPC has an abstraction layer
 for this and thus should be able to work with any toolkit.
 
@@ -1188,7 +658,7 @@ for this and thus should be able to work with any toolkit.
 =item B<loop>
 
 This option takes an object of the loop abstraction layer you
-want to user. Currently the following modules are implemented:
+want to use. Currently the following modules are implemented:
 
   Event::RPC::Loop::Event     Use the Event module
   Event::RPC::Loop::Glib      Use the Glib module
@@ -1208,10 +678,78 @@ Event::RPC will enter the correspondent main loop for you. If you want
 to have full control over the main loop, use this method to setup
 all necessary Event::RPC listeners:
 
-  $server->setup_listeners();
+  $rpc_server->setup_listeners();
 
 and manage the main loop stuff on your own.
 
+=head2 MISCELLANEOUS OPTIONS
+
+=over 4
+
+=item B<host>
+
+By default the network listeners are bound to all interfaces
+in the system. Use the host option to bind to a specific
+interface, e.g. "localhost" if you efficently want to prevent
+network clients from accessing your server.
+
+=item B<auto_reload_modules>
+
+If this option is set Event::RPC::Server will check on each
+method call if the corresponding module changed on disk and
+reloads it automatically. Of course this has an effect on
+performance, but it's very useful during development. You
+probably shouldn't enable this in production environments.
+
+=item B<connection_hook>
+
+This callback is called on each connection / disconnection
+with two arguments: the Event::RPC::Connection object and
+a string containing either "connect" or "disconnect" depending
+what's currently happening with this connection.
+
+=head1 METHODS
+
+The following methods are publically available:
+
+=over 4
+
+=item Event::RPC::Server->B<instance>
+
+This returns the latest created Event::RPC::Server
+instance (usually you have only one instance in one program).
+
+=item $rpc_server->B<start>
+
+Start the mainloop of your Event::RPC::Server.
+
+=item $rpc_server->B<stop>
+
+Stops the mainloop which usually means, that the server exits,
+as long you don't do more sophisticated mainloop stuff by your own.
+
+=item $rpc_server->B<setup_listeners>
+
+This method initializes all networking listeners needed for
+Event::RPC::Server to work, using the configured loop module.
+Use this method if you don't use the start() method but manage
+the mainloop on your own.
+
+=item $rpc_server->B<log> ( [$level,] $msg )
+
+Convenience method for logging. It simply passes the arguments
+to the configured logger's log() method.
+
+=item $rpc_server->B<get_clients_connected>
+
+Returns the number of currently connected Event::RPC clients.
+
+=item $rpc_server->B<get_log_clients_connected>
+
+Returns the number of currently connected logging clients.
+
+=back
+
 =head1 AUTHORS
 
   Jrn Reder <joern at zyn dot de>
